@@ -3,11 +3,10 @@
 
 from trytond.i18n import gettext
 from trytond.model import fields
-from trytond.pool import PoolMeta
-from trytond.pyson import Eval, Bool
-
 from trytond.modules.product import uom_conversion_digits
 from trytond.modules.product.exceptions import UOMValidationError
+from trytond.pool import PoolMeta
+from trytond.pyson import Bool, Eval
 
 
 class PurchaseSecondaryMixin:
@@ -16,15 +15,13 @@ class PurchaseSecondaryMixin:
         'product.uom', "Purchase Secondary UOM",
         domain=[
             ('category', '!=', Eval('default_uom_category')),
-            ],
-        depends=['default_uom_category'])
+            ])
     purchase_secondary_uom_factor = fields.Float(
         "Purchase Secondary UOM Factor", digits=uom_conversion_digits,
         states={
             'required': Bool(Eval('purchase_secondary_uom')),
             'invisible': ~Eval('purchase_secondary_uom'),
             },
-        depends=['purchase_secondary_uom'],
         help="The coefficient for the formula:\n"
         "1 (purchase unit) = coefficient (secondary unit)")
     purchase_secondary_uom_rate = fields.Float(
@@ -33,9 +30,13 @@ class PurchaseSecondaryMixin:
             'required': Bool(Eval('purchase_secondary_uom')),
             'invisible': ~Eval('purchase_secondary_uom'),
             },
-        depends=['purchase_secondary_uom'],
         help="The coefficient for the formula:\n"
         "coefficient (purchase unit) = 1 (secondary unit)")
+    purchase_secondary_uom_category = fields.Function(
+        fields.Many2One(
+            'product.uom.category', "Purchase Secondary UOM Category"),
+        'on_change_with_purchase_secondary_uom_category',
+        searcher='search_purchase_secondary_uom_category')
 
     @fields.depends('purchase_secondary_uom_factor')
     def on_change_purchase_secondary_uom_factor(self):
@@ -55,6 +56,16 @@ class PurchaseSecondaryMixin:
                 1. / self.purchase_secondary_uom_rate,
                 uom_conversion_digits[1])
 
+    @fields.depends('purchase_secondary_uom')
+    def on_change_with_purchase_secondary_uom_category(self, name=None):
+        if self.purchase_secondary_uom:
+            return self.purchase_secondary_uom.category.id
+
+    @classmethod
+    def search_purchase_secondary_uom_category(cls, name, clause):
+        return [('purchase_secondary_uom.category' + clause[0].lstrip(name),)
+            + tuple(clause[1:])]
+
     @property
     def purchase_secondary_uom_normal_rate(self):
         uom = self.purchase_secondary_uom
@@ -86,21 +97,28 @@ class PurchaseSecondaryMixin:
         return factor
 
     @classmethod
-    def validate(cls, records):
-        super().validate(records)
-        for record in records:
-            record.check_purchase_secondary_uom_factor_and_rate()
+    def validate_fields(cls, records, field_names):
+        super().validate_fields(records, field_names)
+        cls.check_purchase_secondary_uom_factor_and_rate(records, field_names)
 
-    def check_purchase_secondary_uom_factor_and_rate(self):
-        factor = self.purchase_secondary_uom_factor
-        rate = self.purchase_secondary_uom_rate
-        if factor and rate:
-            if ((rate != round(1. / factor, uom_conversion_digits[1]))
-                    and factor != round(1. / rate, uom_conversion_digits[1])):
-                raise UOMValidationError(
-                    gettext('purchase_secondary_unit'
-                        '.msg_secondary_uom_incompatible_factor_rate',
-                        record=self))
+    @classmethod
+    def check_purchase_secondary_uom_factor_and_rate(
+            cls, records, field_names):
+        if field_names and not (field_names & {
+                    'purchase_secondary_uom_factor',
+                    'purchase_secondary_uom_rate'}):
+            return
+        for record in records:
+            factor = record.purchase_secondary_uom_factor
+            rate = record.purchase_secondary_uom_rate
+            if factor and rate:
+                new_rate = round(1. / factor, uom_conversion_digits[1])
+                new_factor = round(1. / rate, uom_conversion_digits[1])
+                if rate != new_rate and factor != new_factor:
+                    raise UOMValidationError(
+                        gettext('purchase_secondary_unit'
+                            '.msg_secondary_uom_incompatible_factor_rate',
+                            record=record.rec_ame))
 
 
 class Template(PurchaseSecondaryMixin, metaclass=PoolMeta):
@@ -110,10 +128,8 @@ class Template(PurchaseSecondaryMixin, metaclass=PoolMeta):
     def __setup__(cls):
         super().__setup__()
         cls.purchase_secondary_uom.states = {
-            'readonly': ~Eval('active', True),
             'invisible': ~Eval('purchasable', False),
             }
-        cls.purchase_secondary_uom.depends += ['active', 'purchasable']
 
 
 class Product(metaclass=PoolMeta):
